(require [kodhy.macros [cbind getl λ ss]])

(import
  re
  [matplotlib.pyplot :as plt]
  [kodhy.util [T F R-run R-call ucfirst]])

(defn indicator-model [y x]
  (setv d (.dropna (cbind x :y y) :axis 0 :how "any"))

  ; Rescale.
  (setv scales {})
  (for [col d.columns]
    (when (in (getl d.dtypes col) ["int" "float"])
      (setv (get scales col) (* 2 (.std :ddof 0 (getl d : col))))
      (/= (getl d : col) (get scales col))))

  (R-run "indicator_model = function(d, random.effects = T)
    {library(lme4)
     library(optimx)
     theformula = paste('y ~', paste(collapse = ' + ',
         setdiff(colnames(d), c('y', 's'))))
     if (random.effects)
         theformula = paste(theformula, '+ (1|s)')
     theformula = as.formula(theformula)
     print(theformula)
     m = (
         if (random.effects)
             glmer(theformula, data = d,
                 family = binomial(link = 'logit'), nAGQ = 9,
                 control = glmerControl(optimizer = 'optimx',
                     optCtrl = list(method = 'nlminb')))
         else
             glm(theformula, data = d,
                 family = binomial(link = 'logit')))
     print(summary(m))
     coefs = if (random.effects) fixef(m) else coef(m)
     #list(names(coefs), cbind(coefs, lo = -1, hi = 1))
     cis = confint(m, parm = names(coefs), method = 'boot')
     list(names(coefs), cbind(coefs, cis))}")

  (setv [coef-names coefcis] (R-call "indicator_model" d))
  (setv output (pd.DataFrame coefcis :index coef-names :columns ["point" "lo" "hi"]))
  (print (R-run "warnings()"))
  [output d scales])

(defn go-indicator [d hiv-status xvars yvar yval]
  (cache (.format "indicator-model {} {}" hiv-status yvar) (do
    (setv d2 (ss d (= $hiv_status hiv-status)))
    (indicator-model (= (getl d2 : yvar) yval) (getl d2 : xvars)))))

(defn digest-indicator [from-go]
  (setv [coefcis da scales] from-go)
  ; Reverse the scaling.
  (for [[k scale-factor] (.items scales)]
    (/= (getl coefcis k :) scale-factor))
  ; Antilog the log odds ratios.
  (setv coefcis (np.exp coefcis))
  ; Reorder the rows.
  (setv coefcis (getl coefcis (sorted coefcis.index
    :key (λ (.startswith it "drug_")))))
  ; Prettify the coefficient names.
  (setv coefcis.index (rmap [s coefcis.index] (->> s
    (rx.sub "TRUE$" "")
    (rx.sub "race_([a-z])" (λ (+ "Race: " (ucfirst (.group it 1)))))
    (rx.sub "used_prep" "Used PrEP")
    (rx.sub "age_at_baseline" "Age at baseline (years)")
    (rx.sub "years_since_baseline" "Time elapsed since baseline (years)")
    (rx.sub "years2_since_baseline" "Time elapsed since baseline (years squared)")
    (rx.sub "alcohol_heavy" "heavy drinking")
    (rx.sub "alkyl_nitrite" "nitrite")
    (rx.sub "meth" "methamphetamine")
    (rx.sub "drug_sr_(.+?)([[:upper:]].+)" (λ
      (.format "Drug use: {}: {}"
        (ucfirst (.group it 1))
        (ucfirst (.group it 2))))))))
  coefcis)

(defn table-indicator [from-go]
  ; Reformat the data into a single Series.
  (.apply :axis 1 (digest-indicator from-go) (λ
    (.format "{:.2f} [{:.2f}, {:.2f}]" #* it))))

(defn plot-indicator [from-go &optional drugs-only]
  (setv coefcis (.drop (digest-indicator from-go) :index ["(Intercept)"]))
  (setv coefcis (.reindex coefcis (reversed coefcis.index)))
  (plt.axvline :x 1 :color "black" :linewidth .5 :dashes [3 3])

  (setv drug-type-was None)
  (setv bump 0)
  (setv yticks [])
  (for [i (range (len coefcis))]

    (setv mo (re.match r"Drug use: (.+?):" (get coefcis.index i)))
    (when (and drugs-only (not mo))
      (continue))
    (setv drug-type (when mo (.group mo 0)))
    (when (and drug-type-was (!= drug-type-was drug-type))
      (+= bump 1))
    (setv drug-type-was drug-type)
    
    (.append yticks (,
      (+ i bump)
      ((if drugs-only (λ (cut it (len "Drug use: "))) identity) (get coefcis.index i))))
    (plt.plot [(geti coefcis i 1) (geti coefcis i 2)] [(+ i bump) (+ i bump)]
      :color "black")
    (plt.plot [(geti coefcis i 0)] [(+ i bump)] :color "red" :marker "x"))

  (plt.yticks #* (zip #* yticks) :ha "left")
  (.set-tick-params (.get-yaxis (plt.gca)) :pad 250)
  (plt.semilogx)
  (plt.minorticks-off)
  (setv xticks [.05 .1 .2 .3 .5 .7 1 1.5 2 3 5 10 20 40])
  (plt.xticks xticks (map str xticks))
  None)
