(import
  rpy2.rinterface)

(setv drug-interval-months 6)
  ; The length of the time interval for which the subjects were
  ; asked to report drug use.
(setv mean-days-in-month 30.436875)
  ; http://math.stackexchange.com/questions/1961090
(setv cigarettes-in-pack 20)
  ; Seemingly usual for the United States.

(unless (in "setup_R" (globals))
  (rpy2.rinterface.set-initoptions (tuple (amap
    (.encode it "ASCII") ["rpy2" "--vanilla" "--quiet"])))
  (import rpy2.robjects)
  (import [rpy2.robjects [numpy2ri pandas2ri]])
  (import [rpy2.robjects :as robjects])
  (pandas2ri.activate)
  (setv R rpy2.robjects.r)
  (R "library(Matrix)")
  (R "library(lme4)")
  (setv setup_R T))

(setv evaled-R (set))
(defn eval-R-once [s]
  (unless (in s evaled-R)
    (R s)
    (.add evaled-R s)))

(defn text-freq-to-monthly-freq [x]
  (setv f text-freq-to-monthly-freq)
  (if (pd.isnull x) np.nan (ecase x
    ["Never"                  0]
    ["Once"                   (/ 1 drug-interval-months)]
    ["Rarely"                 (np.mean [(f "Once") (f "Monthly")])]
    ["Less than monthly"      (f "Rarely")]
    ["Monthly or less"        (np.mean [(f "Rarely") (f "Monthly")])]
    ["Monthly"                1]
    ["2 to 4 times a month"   3]
    ["Weekly"                 (/ (f "Daily") 7)]
    ["2 to 3 times a week"    (* 2.5 (f "Weekly"))]
    ["4 or more times a week" (np.mean [(f "2 to 3 times a week") (f "Daily")])]
    ["Occasionally(<1 cigarette/day)" (f "4 or more times a week")]
    ["Daily or almost daily"  (np.mean [(f "4 or more times a week") (f "Daily")])]
    ["Daily"                  mean-days-in-month]
    ["Less than half pack/day"
      (* mean-days-in-month .25 cigarettes-in-pack)]
    ["At least half pack, but less than 1 pack/day"
      (* mean-days-in-month .5 cigarettes-in-pack)]
    ["1 pack/day or more, but less than 2 packs/day"
      (* mean-days-in-month 1 cigarettes-in-pack)]
    ["2+ packs/day"
      (* mean-days-in-month 2 cigarettes-in-pack)]
    ["refuse to answer"       np.nan])))

(defn mse [v1 v2]
  (np.mean (** (- v1 v2) 2)))

(defmacro logistic-model [f]
  `(fn [x-train y-train x-test]
    (setv m ~f)
    (setv x-train (.as-matrix (.drop x-train "s" :axis 1)))
    (.fit m x-train y-train)
    (setv p (.predict-proba m (.as-matrix (.drop x-test "s" :axis 1))))
    {"ypp" (geta p : 1) "coef" (np.append [m.intercept_] m.coef_)}))

(setv models [
  ["trivial-between" (fn [x-train y-train x-test]
    {"ypp" (* [(np.mean y-train)] (len x-test)) "coef" None})]
  ["trivial-within" (fn [x-train y-train x-test]
    {"coef" None "ypp" (rmap [s ($ x-test s)]
      (setv ix (= ($ x-train s) s))
      (if (.any ix)
        (np.mean [(np.mean y-train) (np.mean (get y-train ix.values))])
        (np.mean y-train)))})]
  ["logistic" (logistic-model
    (sklm.LogisticRegression :C 1e10))]
  ["logistic-ridge" (logistic-model
    (sklm.LogisticRegressionCV :Cs 10 :penalty "l2"))]
  ["logistic-lasso" (logistic-model
    (sklm.LogisticRegressionCV :Cs 10 :penalty "l1" :solver "liblinear"))]
  ["logistic-random" (fn [x-train y-train x-test]
    (eval-R-once (.replace "logistic_random = function(d.train, d.test)
       {m = glmer(y ~ � + (1|s), data = d.train, family = binomial)
        ypp = predict(m, d.test, type = 'response',
            allow.new.levels = TRUE)}"
      "�" (.join " + " (filt (!= it "s") x-train.columns))))
    (setv d-train (cbind (.reset-index x-train :drop T) :y y-train))
    {"ypp" ((R "logistic_random") d-train x-test) "coef" None})]])

(defn pred [outcome-var true-val drug-coding-mode obs]

  (setv d (.copy obs))
  ;(setv d (ss d (~ (.isin $s s-weird-interview-dates))))
  (setv d (.reset-index
    (.dropna d :subset [outcome-var])
    :drop T))
  (setv d (.apply (.groupby d "s") (λ
    (setv ($ it t) (. (-
      ($ it interviewed)
      (.min ($ it interviewed))) dt days))
    it)))
  (setv d-base (cbind
    :y (.astype (= (getl d : outcome-var) true-val) int)
    ($ d s)
    (hzscore ($ d t))
    ($ d race_white)
    ($ d race_black)))
  (assert (.all (.all (pd.notnull d-base))))
  (setv d-base (cbind d-base
    (.map ($ d race_hispanic) {"No" 0 "Yes" 1 np.nan np.nan})
    (hzscore ($ d age_at_baseline))))
  (setv y ($ d-base y))

  (setv drug-cols (qw
    drug_sr_alcohol drug_sr_alcohol_heavy drug_sr_cigarettes
    drug_sr_cannabis drug_sr_meth drug_sr_alkyl_nitrite
    drug_sr_powder_cocaine drug_sr_pde5_inhib))
  (setv drug (getl d : drug-cols))
  (setv d-new (cbind d-base (.applymap drug (ecase drug-coding-mode
    ["freq"
      text-freq-to-monthly-freq]
    ["any_or_none"
      (λ (cond [(pd.isnull it) it] [(= it "Never") 0] [True 1]))]))))
  ; Fill missing values with the column mode.
  (setv d-new (.fillna d-new (geti (.mode d-new) 0)))
  ; In frequency mode, standardize the drug variables.
  (when (= drug-coding-mode "freq")
    (for [c drug-cols]
      (setv (getl d-new : c) (hzscore (getl d-new : c)))))

  (setv output {"ypp" {} "coefs_all_obs" {} "cis_all_obs" {}
    "y" y
    "x_vars" (filt (!= it "y") d-new.columns)})

  (setv (get output "results_table") (pd.concat (rmap [[model-name model-f] models]
    (setv cache-key (.format "pred {} {} {} {} {}"
      outcome-var model-name drug-coding-mode (len y)
      (.hexdigest (hashlib.md5 (.encode (.to-json obs) "ASCII")))))
    (setv ypp (pd.Series (cache cache-key (do
      (print model-name :end " ")
      (setv ypp (rmap [ri d-base.index]
        (when 1
          (print ri :end " ")
          (.flush sys.stdout))
        (setv test (geti d-new [ri]))
        (setv train (ss d-new (|
          (!= $s (first (getl test : "s")))
          (< $t (first (getl test : "t"))))))
        (setv y-train (. ($ train y) values))
        (setv x-train (.drop train "y" :axis 1))
        (setv x-test (.drop test "y" :axis 1))
        (get (model-f x-train y-train x-test) "ypp" 0)))
      (print)
      ypp))))
    (setv (get output "ypp" model-name) ypp)
    (unless (= model-name "logistic-random")
      (setv (get output "coefs_all_obs" model-name) (get
        (model-f
          (.drop d-new "y" :axis 1) ($ d-new y) (.drop d-new "y" :axis 1))
        "coef")))
    (when (= model-name "logistic")
      (eval-R-once "logistic_plain_cis = function(d)
          confint(level = .95, glm(y ~ ., data = d,
              family = binomial(link = 'logit')))")
      (setv (get output "cis_all_obs" model-name) ((R "logistic_plain_cis")
        (.drop d-new "s" :axis 1))))

    (pd.DataFrame :index [model-name]
      :columns ["MSE" "p when y=0" "p when y=1"] [[
      (mse ypp y)
      (.mean (get ypp (= y 0)))
      (.mean (get ypp (= y 1)))]]))))

  output)

(defn show-pred-table [pred-output]
  (setv d (.copy (get pred-output "results_table")))
  (for [c d.columns]
    (setv (getl d : c) (rd (if (= c "MSE") 5 3) (getl d : c))))
  d)

(defn pretty-pred-tables [results-freq results-binary]
  (defn f [d]
    (setv d (.copy d))
    (setv d.index (amap (+ "    " (.title (if (= it "logistic") "logistic-plain" it))) d.index))
    d)
  (setv tc ["trivial-between" "trivial-within"])
  (setv d (pd.concat [
    (pd.DataFrame :index ["Trivial"])
    (f (getl (get results-freq "results_table") tc))
    (pd.DataFrame :index ["Dose-response"])
    (f (.drop (get results-freq "results_table") tc))
    (pd.DataFrame :index ["Any-or-none"])
    (f (.drop (get results-binary "results_table") tc))]))
  (for [c d.columns]
    (setv (getl d : c) (rd (if (= c "MSE") 5 3) (getl d : c))))
  (setv d.columns ["MSE" "$p$ when $y=0$" "$p$ when $y=1$"])
  (setv d.index.name "Model")
  d)
